VehicleColor = {
	specName = string.format("%s.vehicleColor", g_currentModName),
	specEntryName = string.format("spec_%s.vehicleColor", g_currentModName)
}

function VehicleColor.prerequisitesPresent(specializations)
	return true
end

function VehicleColor.initSpecialization()
	local schemaSavegame = Vehicle.xmlSchemaSavegame

	schemaSavegame:register(XMLValueType.STRING, string.format("vehicles.vehicle(?).%s.configuration(?)#name", VehicleColor.specName), "Configuration name")
	schemaSavegame:register(XMLValueType.VECTOR_3, string.format("vehicles.vehicle(?).%s.configuration(?)#color", VehicleColor.specName), "Configuration color")
	schemaSavegame:register(XMLValueType.INT, string.format("vehicles.vehicle(?).%s.configuration(?)#material", VehicleColor.specName), "Configuration material")
end

function VehicleColor.registerFunctions(vehicleType)
	SpecializationUtil.registerFunction(vehicleType, "setCustomConfigurationsData", VehicleColor.setCustomConfigurationsData)
	SpecializationUtil.registerFunction(vehicleType, "getCustomConfigurations", VehicleColor.getCustomConfigurations)
	SpecializationUtil.registerFunction(vehicleType, "getCustomConfigurationByName", VehicleColor.getCustomConfigurationByName)
	SpecializationUtil.registerFunction(vehicleType, "getCustomConfigurationsDataIsEqual", VehicleColor.getCustomConfigurationsDataIsEqual)
end

function VehicleColor.registerEventListeners(vehicleType)
	SpecializationUtil.registerEventListener(vehicleType, "onPreLoad", VehicleColor)
	SpecializationUtil.registerEventListener(vehicleType, "onReadStream", VehicleColor)
	SpecializationUtil.registerEventListener(vehicleType, "onWriteStream", VehicleColor)
end

function VehicleColor:onPreLoad(savegame)
	local spec = self[VehicleColor.specEntryName]

	spec.configs = {}

	if savegame ~= nil then
		local i = 0

		while true do
			local key = string.format("%s.%s.configuration(%d)", savegame.key, VehicleColor.specName, i)

			if not savegame.xmlFile:hasProperty(key) then
				break
			end

			local name = savegame.xmlFile:getValue(key .. "#name")
			local r, g, b = savegame.xmlFile:getValue(key .. "#color")
			local material = savegame.xmlFile:getValue(key .. "#material")

			spec.configs[name] = {
				color = {r, g, b},
				material = material
			}

			i = i + 1
		end
	end
end

function VehicleColor:saveToXMLFile(xmlFile, key, usedModNames)
	local spec = self[VehicleColor.specEntryName]
	local i = 0

	for configName, configData in pairs(spec.configs) do
		local configKey = string.format("%s.configuration(%d)", key, i)

		xmlFile:setValue(configKey .. "#name", configName)
		xmlFile:setValue(configKey .. "#color", configData.color[1], configData.color[2], configData.color[3])
		xmlFile:setValue(configKey .. "#material", configData.material)

		i = i + 1
	end
end

function VehicleColor:setCustomConfigurationsData(colorConfiguratorData)
	if colorConfiguratorData ~= nil then
		local spec = self[VehicleColor.specEntryName]
		local customConfigActive = false

		spec.configs = {}

		for configName, configData in pairs(colorConfiguratorData) do
			spec.configs[configName] = configData

			if configName == "baseMaterial" or configName == "designMaterial" or configName == "designMaterial2" or configName == "designMaterial3" then
				self:applyBaseMaterialConfiguration(self.xmlFile, configName, self.configurations[configName])
			end

			ConfigurationUtil.applyConfigMaterials(self, self.xmlFile, configName, self.configurations[configName])

			if configName == "baseColor" or configName == "designColor" then
				ConfigurationUtil.setColor(self, self.xmlFile, configName, self.configurations[configName])
			end

			if configName == "wrappingColor" then
				self:applyBaseMaterialConfiguration(self.xmlFile, configName, self.configurations[configName])
			end

			customConfigActive = true
		end

		if customConfigActive then
			ColorConfiguratorUtil.updateConfigurations(self)

			if self.applyBaseMaterial ~= nil then
				self:applyBaseMaterial()
			end
		end
	end
end

function VehicleColor:getCustomConfigurations()
	return self[VehicleColor.specEntryName].configs
end

function VehicleColor:getCustomConfigurationByName(configName)
	local configs = self:getCustomConfigurations()

	if configs ~= nil then
		return configs[configName]
	end

	return nil
end

function VehicleColor:onReadStream(streamId, connection)
	if connection:getIsServer() then
		local spec = self[VehicleColor.specEntryName]
		local data = {}
		local numConfigs = streamReadUInt8(streamId)

		for _ = 1, numConfigs do
			local configName = streamReadString(streamId)
			local r = streamReadFloat32(streamId)
			local g = streamReadFloat32(streamId)
			local b = streamReadFloat32(streamId)
			local material = streamReadInt32(streamId)

			data[configName] = {
				color = ColorConfiguratorUtil.formatColor(r, g, b),
				material = material
			}
		end

		self:setCustomConfigurationsData(data)
	end
end

function VehicleColor:onWriteStream(streamId, connection)
	if not connection:getIsServer() then
		local spec = self[VehicleColor.specEntryName]
		local numConfigs = 0

		for _, _ in pairs(spec.configs) do
			numConfigs = numConfigs + 1
		end

		streamWriteUInt8(streamId, numConfigs)

		for configName, configData in pairs(spec.configs) do
			streamWriteString(streamId, configName)
			streamWriteFloat32(streamId, configData.color[1])
			streamWriteFloat32(streamId, configData.color[2])
			streamWriteFloat32(streamId, configData.color[3])
			streamWriteInt32(streamId, configData.material)
		end
	end
end

function VehicleColor:getCustomConfigurationsDataIsEqual(data)
	if data ~= nil then
		for configName, configData in pairs(data) do
			local ownData = self:getCustomConfigurationByName(configName)

			if ownData ~= nil then
				if ownData.material ~= configData.material or not table.equals(ownData.color, configData.color) then
					return false
				end
			end
		end
	end

	return true
end